package com.qmth.teachcloud.report.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.report.business.bean.dto.query.BasicExamRecordDto;
import com.qmth.teachcloud.report.business.bean.result.*;
import com.qmth.teachcloud.report.business.entity.TAExamCourse;
import com.qmth.teachcloud.report.business.entity.TAExamCourseRecord;
import com.qmth.teachcloud.report.business.entity.TBPaper;
import com.qmth.teachcloud.report.business.enums.PublishStatusEnum;
import com.qmth.teachcloud.report.business.enums.SemesterEnum;
import com.qmth.teachcloud.report.business.mapper.TAExamCourseMapper;
import com.qmth.teachcloud.report.business.mapper.TAExamCourseRecordMapper;
import com.qmth.teachcloud.report.business.service.*;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 考试课程维度分析表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-06-01
 */
@Service
public class TAExamCourseServiceImpl extends ServiceImpl<TAExamCourseMapper, TAExamCourse> implements TAExamCourseService {

    @Resource
    TAExamCourseMapper taExamCourseMapper;

    @Resource
    TAExamCourseRecordMapper taExamCourseRecordMapper;

    @Resource
    TBPaperStructService tbPaperStructService;

    @Resource
    ReportCommonService reportCommonService;

    @Resource
    TBExamRecordService tbExamRecordService;

    @Resource
    TBExamCourseService tbExamCourseService;

    @Resource
    TBExamService tbExamService;

    @Resource
    BasicCourseService basicCourseService;

    /**
     * 开课课程考试总览列表接口
     *
     * @param iPage
     * @param examId
     * @param semester
     * @param schoolId
     * @return
     */
    @Override
    public IPage<TAExamCourseResult> surveyTeacherList(IPage<Map> iPage, Long examId, SemesterEnum semester, Long schoolId) {
        return taExamCourseMapper.surveyTeacherList(iPage, examId, semester.name(), schoolId);
    }


    @Override
    public ExamCourseResult getOverview(String semester, Long examId, String courseCode) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return taExamCourseMapper.getOverview(schoolId, semester, examId, courseCode);
    }

    /**
     * 赋分管理列表接口
     *
     * @param iPage
     * @param schoolId
     * @param examId
     * @param courseCode
     * @param publishStatus
     * @return
     */
    @Override
    public IPage<TAExamCourseResult> surveyAspointsList(IPage<Map> iPage, Long schoolId, Long examId, String courseCode, PublishStatusEnum publishStatus) {
        return taExamCourseMapper.surveyAspointsList(iPage, schoolId, examId, courseCode, publishStatus.name());
    }

    @Override
    public Map<String, Object> getGradeDistribute(String semester, Long examId, String courseCode) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Map<String, Object> map = new HashMap<>();
        // 不及格率
        Map<String, String> scoreAndRate = taExamCourseMapper.getScoreAndRate(schoolId, semester, examId, courseCode);
        map.put("scoreAndRate", scoreAndRate);
        // 成绩正态分布
        String[] strings = {"90~100", "80~89", "70~79", "60~69", "55~59", "1~54"};
        List<Map<String, Object>> mapList = new ArrayList<>();
        for (String string : strings) {
            String[] str = string.split("~");
            double startScore = Double.parseDouble(str[0]);
            double endScore = Double.parseDouble(str[1]);
            List<TAExamCourseRecord> taExamCourseRecords = taExamCourseRecordMapper.listExamCourseRecord(schoolId, semester, examId, courseCode);
            // 全部有效人数
            long totalRealityCount = taExamCourseRecords.stream().filter(m -> Objects.nonNull(m.getAbsent()) && !m.getAbsent()).count();
            // 本分数段人数
            long totalScoreCount = taExamCourseRecords.stream().filter(m -> m.getAssignedScore().doubleValue() >= startScore && m.getAssignedScore().doubleValue() <= endScore).count();
            // 应届有效人数
            long totalCurrentCount = taExamCourseRecords.stream().filter(m -> Objects.nonNull(m.getAbsent()) && !m.getAbsent() && Objects.nonNull(m.getStudentCurrent()) && m.getStudentCurrent()).count();
            // 本分数段应届有效人数
            long totalScoreCurrentCount = taExamCourseRecords.stream().filter(m -> m.getStudentCurrent() && m.getAssignedScore().doubleValue() >= startScore && m.getAssignedScore().doubleValue() <= endScore).count();
            Map<String, Object> objectMap = new HashMap<>();
            objectMap.put("scores", String.join(",", str));
            objectMap.put("totalRealityCount", totalRealityCount);
            objectMap.put("totalRate", totalRealityCount == 0 ? 0 : new BigDecimal(totalScoreCount * 100).divide(new BigDecimal(totalRealityCount), 1, BigDecimal.ROUND_HALF_UP));
            objectMap.put("currentRealityCount", totalCurrentCount);
            objectMap.put("currentRate", totalCurrentCount == 0 ? 0 : new BigDecimal(totalScoreCurrentCount * 100).divide(new BigDecimal(totalCurrentCount), 1, BigDecimal.ROUND_HALF_UP));
            mapList.add(objectMap);
        }
        map.put("grades", mapList);
        return map;
    }

    /**
     * 考查课程考试分析接口
     *
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.COLLEGE_COURSE_REPORT, key = "#schoolId + '-' + #semester + '-' + #examId + '-' + #courseCode")
    public CollegeAndCourseResult surveyInspectAnalyseView(Long schoolId, SemesterEnum semester, Long examId, String courseCode) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (Objects.isNull(sysUser)) {
            throw ExceptionResultEnum.NOT_LOGIN.exception();
        }
        //学院学科报表查询科目信息
        CourseInfoResult courseInfoResult = reportCommonService.findCourseInfo(examId, courseCode, sysUser.getOrgId());
        //学院学科报表科目描述
        CollegeAndSchoolGradeDistributionResult collegeAndSchoolGradeDistributionResult = reportCommonService.findCourseDistribution(examId, courseCode, sysUser.getOrgId());
        //学院学科报表维度查询
        DimensionAnalyzeResult dimensionAnalyzeResult = reportCommonService.findDimensionInfo(examId, courseCode, sysUser.getOrgId(), null);
        //查找试卷结构
        List<TBPaperStructResult> questionDatasource = tbPaperStructService.findQuestionInfo(examId, courseCode);
        //查找题目相关
        QuestionInfoResult questionInfoResult = reportCommonService.findSituationOfQuestions(examId, courseCode, sysUser.getOrgId(), questionDatasource);
        //查找老师得分
        TeacherInfoListResult teacherInfoListResult = reportCommonService.findTeacherInfo(examId, courseCode, sysUser.getOrgId());
        //查找题目信息
        QuestionListResult questionListResult = reportCommonService.findQuestionInfo(examId, courseCode, sysUser.getOrgId(), questionDatasource);

        return new CollegeAndCourseResult(courseInfoResult, collegeAndSchoolGradeDistributionResult, dimensionAnalyzeResult,
                questionInfoResult, teacherInfoListResult, questionListResult);
    }

    /**
     * 开课课程考试总览图表接口
     *
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SURVEY_TEACHER_VIEW_REPORT, key = "#schoolId + '-' + #semester + '-' + #examId + '-' + #courseCode")
    public SurveyTeacherViewResult surveyTeacherView(Long schoolId, SemesterEnum semester, Long examId, String courseCode) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (Objects.isNull(sysUser)) {
            throw ExceptionResultEnum.NOT_LOGIN.exception();
        }
        //取总体和应届平均分
        QueryWrapper<TAExamCourse> taExamCourseQueryWrapper = new QueryWrapper<>();
        if (Objects.nonNull(schoolId)) {
            taExamCourseQueryWrapper.lambda().eq(TAExamCourse::getSchoolId, schoolId);
        }
        if (Objects.nonNull(examId)) {
            taExamCourseQueryWrapper.lambda().eq(TAExamCourse::getExamId, examId);
        }
        if (Objects.nonNull(courseCode)) {
            taExamCourseQueryWrapper.lambda().eq(TAExamCourse::getCourseCode, courseCode);
        }
        TAExamCourse taExamCourse = this.getOne(taExamCourseQueryWrapper);
        BigDecimal bigDecimal = new BigDecimal(1);
        SurveyTeacherExamCourseResult surveyTeacherExamCourseResult = new SurveyTeacherExamCourseResult(taExamCourse.getAvgScore(), bigDecimal.subtract(taExamCourse.getPassRate()), taExamCourse.getPaperCurrentAvgScore(), bigDecimal.subtract(taExamCourse.getPaperPassRate()));

        //学院学科报表科目描述
        List<SurveyTeacherGradeDistributionResult> surveyTeacherGradeDistributionResultList = reportCommonService.surveyTeacherDistribution(examId, courseCode, sysUser.getOrgId());
        //查找试卷结构
        List<TBPaperStructResult> questionDatasource = tbPaperStructService.findQuestionInfo(examId, courseCode);
        //查找题目信息
        QuestionListResult questionListResult = reportCommonService.findQuestionInfo(examId, courseCode, sysUser.getOrgId(), questionDatasource);
        return new SurveyTeacherViewResult(surveyTeacherGradeDistributionResultList, questionListResult, surveyTeacherExamCourseResult);
    }

    /**
     * 开课课程考试总览-教师各课堂成绩排名接口
     *
     * @param iPage
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @return
     */
    @Override
    public IPage<TAExamCourseCollegeTeacherResult> surveyTeacherTeacherView(IPage<Map> iPage, Long schoolId, SemesterEnum semester, Long examId, String courseCode) {
        return taExamCourseMapper.surveyTeacherTeacherView(iPage, schoolId, semester.name(), examId, courseCode);
    }

    /**
     * 开课课程考试总览-试题难度得分情况分析接口
     *
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @param collegeId
     * @param teacherId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.TEACHER_VIEW_REPORT, key = "#schoolId + '-' + #semester + '-' + #examId + '-' + #courseCode + '-' + #collegeId + '-' + #teacherId")
    public DimensionAnalyzeResult surveyTeacherQuestionView(Long schoolId, SemesterEnum semester, Long examId, String courseCode, Long collegeId, Long teacherId) {
        return reportCommonService.findDimensionInfo(examId, courseCode, collegeId, teacherId);
    }

    /**
     * 开课课程考试总览-各难度水平题目上的作答分析接口
     *
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @param collegeId
     * @param teacherId
     * @return
     */
    @Override
    public Object surveyTeacherQuestionAnswerView(Long schoolId, SemesterEnum semester, Long examId, String courseCode, Long collegeId, Long teacherId) {
        //查找试卷结构
        List<TBPaperStructResult> questionDatasource = tbPaperStructService.findQuestionInfo(examId, courseCode);
        //开课课程考试总览-按学院/教师查询
        QuestionInfoResult questionInfoResult = reportCommonService.findSituationOfQuestions(examId, courseCode, collegeId, questionDatasource);
        return questionInfoResult;
    }

    @Override
    public TrialCalculationResult trialCalculate(Long examId, String courseCode) {
        List<String> courseCodeList = new ArrayList<>();
        courseCodeList.add(courseCode);

//        // 数据同步操作
//        // 获取当前课程下所有学生考试成绩记录
//        List<BasicExamRecordDto> basicExamRecordDtoDatasource = tbExamRecordService.findByExamIdAndCourseCodeS(examId, courseCodeList);
//        for (String s : courseCodeList) {
//            if (tbExamCourseService.verifyExamCourseCantRun(examId, tbExamService.getById(examId).getSchoolId(), s, basicCourseService.findByCourseCode(s).getName())) {
//                throw ExceptionResultEnum.ERROR.exception("课程编号[" + s + "]的课程分析数据已测试或发布，不能变更基础数据");
//            }
//            List<BasicExamRecordDto> basicExamRecordDtoList = basicExamRecordDtoDatasource.stream()
//                    .filter(e -> s.equals(e.getCourseCode())).collect(Collectors.toList());
//            if (basicExamRecordDtoList.size() > 0) {
//                // 迁移数据至't_a_exam_course_record'
//                List<TAExamCourseRecord> taExamCourseRecordList = new ArrayList<>();
//                for (BasicExamRecordDto basicExamRecordDto : basicExamRecordDtoList) {
//                    boolean absent = basicExamRecordDto.getAbsent();
//                    // 正常公式赋分操作
//                    Long paperId = basicExamRecordDto.getPaperId();
//                    TBPaper tbPaper = tbPaperService.getById(paperId);
//                    if (Objects.isNull(tbPaper)) {
//                        throw ExceptionResultEnum.ERROR.exception("试卷信息数据异常");
//                    }
//                    BigDecimal fullScore = tbPaper.getTotalScore();
//                    BigDecimal myScore = basicExamRecordDto.getTotalScore();
//                    BigDecimal coefficient = tbPaper.getCoefficient();
//                    BigDecimal assignScore;
//                    // 当该试卷的赋分系数不为0时赋分
//                    if (coefficient != null && coefficient.compareTo(BigDecimal.ZERO) > 0 && !absent) {
//                        assignScore = myScore.add((fullScore.subtract(myScore)).divide(coefficient, 0, BigDecimal.ROUND_HALF_UP));
//                    } else {
//                        assignScore = myScore;
//                    }
//
//                    TAExamCourseRecord taExamCourseRecord = new TAExamCourseRecord();
//                    taExamCourseRecord.setId(SystemConstant.getDbUuid());
//                    taExamCourseRecord.setExamRecordId(basicExamRecordDto.getTbExamRecordId());
//                    // 数据同步默认赋分成绩为卷面成绩
//                    taExamCourseRecord.setSchoolId(tbExamService.getById(examId).getSchoolId());
//                    taExamCourseRecord.setAssignedScore(assignScore);
//                    taExamCourseRecord.setExamId(examId);
//                    taExamCourseRecord.setCourseCode(basicExamRecordDto.getCourseCode());
//                    taExamCourseRecord.setPaperId(basicExamRecordDto.getPaperId());
//                    taExamCourseRecord.setPaperType(basicExamRecordDto.getPaperType());
//                    taExamCourseRecord.setStudentId(basicExamRecordDto.getStudentId());
//                    taExamCourseRecord.setStudentCode(basicExamRecordDto.getStudentCode());
//                    taExamCourseRecord.setTeacherId(basicExamRecordDto.getTeacherId());
//                    taExamCourseRecord.setClazzId(basicExamRecordDto.getClazzId());
//                    taExamCourseRecord.setTeachCollegeId(basicExamRecordDto.getTeachCollegeId());
//                    taExamCourseRecord.setInspectCollegeId(basicExamRecordDto.getInspectCollegeId());
//                    taExamCourseRecord.setMajorId(basicExamRecordDto.getMajorId());
//                    taExamCourseRecord.setTotalScore(myScore);
//                    taExamCourseRecord.setAbsent(absent);
//                    taExamCourseRecord.setStudentCurrent(basicExamRecordDto.getStudentCurrent());
//                    taExamCourseRecordList.add(taExamCourseRecord);
//                }
//                taExamCourseRecordService.saveBatch(taExamCourseRecordList);
//            }
//        }
        return null;
    }
}
